/* ======================================================================
OBJECT: 			com_netobjects_ColorPicker

DESCRIPTION:	The ColorPicker object opens a pop-up browser window that
					contains a table full of color blocks. Users can select
					a color block and the corresponding RGB value will be
					returned.  The value can then be retrieved for individual
					red, green and blue values in decimal or hexidecimal 
					representations, or as one combined string in hexidecimal 
					(commonly used in HTML pages).

TESTED
PLATFORMS:	Netscape Navigator 4.05, 
				Microsoft Internet Explorer 4.01
====================================================================== */
function com_netobjects_ColorPicker(params) {
 
   // Public Properties	
	this.name 			= (params.name != null) ? params.name : "ColorPicker1" ;
	this.testMode 		= (params.testMode != null) ? params.testMode : "off";
	this.top 			= (params.top != null) ? params.top : "";
	this.left 			= (params.left != null) ? params.left : "";
	this.pickedColor	= "";  // set by eOnSelColor();
	this.cellDim		= 13;
	this.columns		= 16;

	// Private Properties
	this.hdlWindow		= null;	// handle to the opened window
	this.URL				= ""; 
	this.toolbar 		= false;
	this.location 		= false;
	this.directories 	= false;
	this.status 		= false;
	this.menubar 		= false;
	this.scrollbars 	= false;
	this.resizable 	= true;
	this.width 			= 260;
	this.height 		= 235;
	this.blankImage	= "blank.gif";

	// Public Methods
	this.close			= _colorpicker_close;
	this.open			= _colorpicker_open;  
	this.convertToHex = _colorpicker_convertToHex;
	this.render 		= _colorpicker_render;
	this.getName		= _colorpicker_getName;
	this.setName		= _colorpicker_setName;

	// Private Methods
	this.getFeatures			= _colorpicker_getFeatures;
	this.validateProperties = _colorpicker_validateProperties;

	// Public Events
	this.onClose		= _colorpicker_onClose;
	this.onOpen			= _colorpicker_onOpen;
	this.onRender		= _colorpicker_onRender;
	this.onSelColor	= _colorpicker_onSelColor;

	// Private Events (internal event driven functionality)
	this.eOnClose		= _colorpicker_eOnClose;
	this.eOnSelColor	= _colorpicker_eOnSelColor;

	// If test mode is set, check for bad property values.
	this.validateProperties();
	// Public Methods ----------------------------------------

	/* ======================================================================
	
   METHOD: _colorpicker_render.  Writes out the HTML table of color tiles in
	the opened pop-up window indicated by windowHandle.  The document is written
	out in a brute-force manner, rather than generated dynamically, to improve
	rendering speed in some browsers.
	
	====================================================================== */
	/* ======================================================================
	
   METHOD: _colorpicker_render.  Renders the HTML table that allows users
	to choose a color.
	
	====================================================================== */
	function _colorpicker_render() {	
		var INCREMENT = 51;	// Increment each color value by hex 33
		var rgbValue = 0;
		var doc = this.hdlWindow.document;
		var count = 0;

		doc.clear();
		doc.open();
		doc.writeln("<HTML><BODY BGCOLOR=\"#000000\">");
		doc.writeln("<CENTER><TABLE BORDER=1 CELLPADDING=0 CELLSPACING=0><TR>");
		for (k=0; k <= 255; k += INCREMENT)
			for (j=0; j <= 255; j += INCREMENT)
				for (i=0; i <= 255; i += INCREMENT) {
					if (count != 0 && (count % this.columns == 0))
						doc.writeln("</TR><TR>");
					colorStr = this.convertToHex(k) + this.convertToHex(j) + this.convertToHex(i);
					doc.writeln("<TD BGCOLOR=\"#" + colorStr + "\">" + 
						"<A HREF=\"javascript:window.opener." + this.name + ".eOnSelColor('" + colorStr + "')\"><IMG NAME=\"" + colorStr + "\"ALT=\"#" + colorStr + "\" BORDER=0 SRC=\"" + this.blankImage + "\" WIDTH=" + this.cellDim + " HEIGHT=" + this.cellDim + "></A></TD>");
					count++;
				} // end for loop
				doc.writeln("</TR></TABLE");

		doc.writeln("End<br>");
		doc.writeln("</CENTER></BODY></HTML>");
		doc.close();
		// Fire public event
		this.onRender(); 
	} // END _colorpicker_open

	// Private Methods ----------------------------------------

	/* ======================================================================
	
   METHOD: _colorpicker_open.  Opens the color picker window.
	
	====================================================================== */
	function _colorpicker_open() {	
		// Make sure the window isn't already open before opening it
		if (this.hdlWindow == null || this.hdlWindow.closed) {
			this.hdlWindow = window.open(this.URL, this.name, this.getFeatures());
			this.render(this.hdlWindow);
		}

		// Fire public event
		this.onOpen(); 
	} // END _colorpicker_open


	/* ======================================================================
	
   METHOD: _colorpicker_close.  Closes the color picker window if it's open.
	
	====================================================================== */
	function _colorpicker_close() {	
		err = "";

		if (this.hdlWindow != null && !this.hdlWindow.closed) 
			this.hdlWindow.close();
		
		// Fire private event
		this.eOnClose(); 
	} // END _colorpicker_close


	/* ======================================================================
	
   METHOD: _colorpicker_convertToHex.  Converts an integer to a Hex string.
	
	====================================================================== */
	function _colorpicker_convertToHex( theInt ) {
		var bitmask = 0xF;
		var leftDigit = 0, rightDigit = 0;
		var leftChar = "", rightChar = "";
		var hexDigitArray = new Array("A", "B", "C", "D", "E", "F");

		rightDigit = theInt & bitmask;
		rightChar = (rightDigit > 9) ? rightChar = hexDigitArray[rightDigit-10] : rightDigit+"";

		leftDigit = (theInt >> 4) & bitmask;
		leftChar = (leftDigit > 9)? leftChar = hexDigitArray[leftDigit-10] : leftDigit+"";

		return leftChar + rightChar;
	} // END _colorpicker_convertToHex

	// Property Getters 
	/* ======================================================================
	
   METHOD: _colorpicker_getName.  Getter for the 'name' property.
	
	====================================================================== */
	function _colorpicker_getName() {	
		return this.name;
	} // END _colorpicker_getName


	// Property Setters

	/* ======================================================================
	
   METHOD: _colorpicker_setName.  Setter for the 'name' property.  Returns
	true if the property was set, false if it could not be set.
	
	====================================================================== */
	function _colorpicker_setName(thevalue) {	
		var returnVal = true;

		// Don't allow users to set the name once a name has been set.  Errors
		// will occur if the object's name is changed midstream.
		if (this.name == null && !IsNum(thevalue.charAt(0)) && IsAlphaNumOrUnderscore(thevalue))
			this.name = thevalue;
		else
			returnVal = false;

		return returnVal;
	} // END _colorpicker_setName


	// Private Methods ----------------------------------------

	/* ======================================================================
	
   METHOD: _colorpicker_getFeatures.  Generates a string that combines all 
	all of the yes/no propeties into a comma-delimited list of name/value
   pairs.  This string corresponds to the required syntax of the JavaScript
	window.open() method's feature list argument.

	E.g.  statusbar=yes,toolbar=yes,height=200,width=100
	
	====================================================================== */
	function _colorpicker_getFeatures() {
		sFeatures = "";
		
		if (this.toolbar)
			sFeatures += "toolbar=yes";
		if (this.location) 
			sFeatures += ",location=yes";
		if (this.directories) 
			sFeatures += ",directories=yes";
		if (this.status) 
			sFeatures += ",status=yes";
		if (this.menubar) 
			sFeatures += ",menubar=yes";
		if (this.scrollbars) 
			sFeatures += ",scrollbars=yes";
		if (this.resizable) 
			sFeatures += ",resizable=yes";
		if (this.width != null && this.width != "") 
			sFeatures += ",width=" + this.width;
		if (this.height != null && this.height != "") 
			sFeatures += ",height=" + this.height;
		if (this.top != null && this.left != "") 
			sFeatures += ",top=" + this.top;
		if (this.left != null && this.left != "") 
			sFeatures += ",left=" + this.left;
		
		// Strip off the unnecessary starting comma, if one exists
		if (sFeatures.charAt(0) == ',')
			sFeatures = sFeatures.substring(1, sFeatures.length);

		return sFeatures;
	} // END	_colorpicker_getFeatures()


	/* ======================================================================
	
   METHOD: _colorpicker_validateProperties.  Used only in test mode.  Writes out
	error and warning messages if some property values are set with bad
	values.
	
	====================================================================== */
	function _colorpicker_validateProperties() {
		if ((this.testMode.toLowerCase() == "text") || (this.testMode.toLowerCase() == "alerts")) {
			var errors = new Array();
	      iIndex = 0;  
         
			// Check for errors
			if (this.name == "") 
				errors[iIndex++] = "Name is a required property.";  

			// Display errors
			for (var i=0; i<errors.length; i++)  {
				if (this.testMode.toLowerCase()	== "text") 
  					document.write( "<BR><HR><B><FONT COLOR=RED>TEST MODE, com_netobjects_ColorPicker, '" + this.name + "' : </FONT> " + errors[i] + "</B><HR>\n");
				else
  					alert("TEST MODE, com_netobjects_ColorPicker, '" + this.name + "' : " + errors[i]);
			}			
		} 
	} // END _colorpicker_validateProperties
	

	// Private Event Handlers ----------------------------------------

	/* ======================================================================
	
   EVENT: _colorpicker_eOnClose.  Internal event handler.  Resets window handle
 	to null then fires the public onClose event.
	
	====================================================================== */
	function _colorpicker_eOnClose() {
		this.hdlWindow = null;	

		// Fire public event
		this.onClose();
	} // end _colorpicker_eOnClose()


	/* ======================================================================
	
   EVENT: _colorpicker_eOnClose.  Internal event handler.  Resets window handle
 	to null then fires the public onClose event.
	
	====================================================================== */
	function _colorpicker_eOnSelColor(colorStr) {
		this.pickedColor = colorStr;	

		// Fire public event
		this.onSelColor();
	} // end _colorpicker_eOnClose()

	
	// Public Event Handlers ----------------------------------------

	function _colorpicker_onClose() {
	}
	function _colorpicker_onSelColor() {
	}
	function _colorpicker_onOpen() {
	}
	function _colorpicker_onRender() {
	}
	

} // END CONSTRUCTOR com_netobjects_ColorPicker


/* ======================================================================
FUNCTION:  	IsNum
 
INPUT:  		numstr (string/number) - the string that will be tested to ensure 
      										 that the value is a number (int or float)

RETURN:  	true, if all characters represent a valid integer or float
     			false, otherwise.

PLATFORMS:	Netscape Navigator 3.01 and higher,
			  	Microsoft Internet Explorer 3.02 and higher,
			  	Netscape Enterprise Server 3.0,
			  	Microsoft IIS/ASP 3.0.
====================================================================== */
function IsNum( numstr ) {
	// Return immediately if an invalid value was passed in
	if (numstr+"" == "undefined" || numstr+"" == "null" || numstr+"" == "")	
		return false;

	var isValid = true;
	var decCount = 0;		// number of decimal points in the string

	// convert to a string for performing string comparisons.
	numstr += "";	

	// Loop through string and test each character. If any
	// character is not a number, return a false result.
 	// Include special cases for negative numbers (first char == '-')
	// and a single decimal point (any one char in string == '.').   
	for (i = 0; i < numstr.length; i++) {
		// track number of decimal points
		if (numstr.charAt(i) == ".")
			decCount++;

    	if (!((numstr.charAt(i) >= "0") && (numstr.charAt(i) <= "9") || 
				(numstr.charAt(i) == "-") || (numstr.charAt(i) == "."))) {
       	isValid = false;
       	break;
		} else if ((numstr.charAt(i) == "-" && i != 0) ||
				(numstr.charAt(i) == "." && numstr.length == 1) ||
			  (numstr.charAt(i) == "." && decCount > 1)) {
       	isValid = false;
       	break;
      }         	         	       
   } // END for   
   
   	return isValid;
}  // end IsNum


/* ======================================================================
FUNCTION:	IsAlphaNumOrUnderscore

INPUT:		str (string) - the string to be tested

RETURN:  	true, if the string contains only alphanumeric characters or underscores.
				false, otherwise.

PLATFORMS:	Netscape Navigator 3.01 and higher,
			  	Microsoft Internet Explorer 3.02 and higher,
			  	Netscape Enterprise Server 3.0,
			  	Microsoft IIS/ASP 3.0.
====================================================================== */
function IsAlphaNumOrUnderscore( str ) {
	// Return immediately if an invalid value was passed in
	if (str+"" == "undefined" || str+"" == "null" || str+"" == "")	
		return false;

	var isValid = true;

	str += "";	// convert to a string for performing string comparisons.
	// Loop through string one character at a time. If non-alpha numeric
	// is found then, break out of loop and return a false result

	for (i = 0; i < str.length; i++)
   	{
		// Alphanumeric must be between "0"-"9", "A"-"Z", or "a"-"z"
      		if ( !( ((str.charAt(i) >= "0") && (str.charAt(i) <= "9")) || 
      			((str.charAt(i) >= "a") && (str.charAt(i) <= "z")) ||
      			((str.charAt(i) >= "A") && (str.charAt(i) <= "Z")) ||
      			(str.charAt(i) == "_") ) )
      		{
   				isValid = false;
         		break;
      		}

	} // END for   
   
	return isValid;
}  // end IsAlphaNumOrUnderscore
